import axios from "axios"

interface Options {
  baseUrl?: string
  apiKey: string
}

export interface RatesResp {
  updatedAt: string
  rates: Rates
}

interface Rates {
  [key: string]: number
}

export interface Currencies {
  [key: string]: string
}

export default class OpenExchangeRates {
  #prefixUrl: string
  #apiKey: string

  public constructor(options: Options) {
    this.#prefixUrl = `${options.baseUrl ?? "https://openexchangerates.org"}/api/`
    this.#apiKey = options.apiKey
  }

  private basicRequestParams(): object {
    return {
      app_id: this.#apiKey,
      prettyprint: "false",
      show_alternative: "false",
      show_inactive: "false",
    }
  }

  private async request(path: string, params?: object) {
    let url = this.#prefixUrl + path + ".json"
    let finalParams = { ...this.basicRequestParams(), ...params }
    return await axios.get(url, {
      headers: {
        Accept: "application/json",
      },
      params: finalParams,
    })
  }

  public async usage() {
    const resp = await this.request("usage")
    return resp.data
  }

  public async currencies(): Promise<Currencies> {
    const resp = await this.request("currencies")
    return resp.data
  }

  public async latest(currencies: Array<string>): Promise<RatesResp> {
    const resp = await this.request("latest", {
      base: "USD",
      symbols: currencies.join(","),
    })
    return {
      updatedAt: new Date(resp.data["timestamp"]).toISOString(),
      rates: resp.data["rates"],
    }
  }
}
